import 'package:flutter/widgets.dart';
import 'package:flutter_modular/flutter_modular.dart';

import '../modular_base.dart';

class Inject<R> {
  Map<String, dynamic> params = {};
  final Type tag;

  Inject({
    this.params,
    this.tag,
  });

  factory Inject.of() => Inject();

  ///get injected dependency
  T get<T>([Map<String, dynamic> params]) {
    params ??= {};
    return Modular.getInjectableObject<R, T>(tag, params: params);
  }

  dispose<T>() {
    return Modular.removeInjectableObject<R, T>(tag);
  }
}

mixin InjectMixin<T> {
  final Inject<T> _inject = Inject<T>.of();

  S get<S>() {
    return _inject.get<S>();
  }

  Widget consumer<S extends ChangeNotifier>({
    Widget Function(BuildContext context, S value) builder,
    bool Function(S oldValue, S newValue) distinct,
  }) {
    return ConsumerWidget<S>(
      builder: builder,
      distinct: distinct,
      inject: _inject,
    );
  }
}
